
;**************************************************************************
;*                             SRDIECKO                                   *
;**************************************************************************
;*          8 LED connected to PORTB & 6 to PORTA                         *
;*          1 button can be used to switch between impemented effects     *
;**************************************************************************
;*          v 5.02 - 18.07.2008                                           *
;**************************************************************************
;*          Hardw.: PIC16F628A used                                       *
;*          OSC.......: Internal osc. used     POWER.....:  2x1,2V accu   *
;**************************************************************************

W                            EQU     H'0000'
F                            EQU     H'0001'

;----- Register Files------------------------------------------------------

INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'

INTCON                       EQU     H'000B'
OPTION_REG                   EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
CMCON                        EQU     H'001F'

;----- STATUS Bits --------------------------------------------------------
IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;==========================================================================
;
;       RAM Definition
;
;==========================================================================

    __MAXRAM H'01FF'
    __BADRAM H'07'-H'09', H'0D', H'13'-H'14', H'1B'-H'1E'
    __BADRAM H'87'-H'89', H'8D', H'8F'-H'91', H'93'-H'97', H'9E'
    __BADRAM H'105', H'107'-H'109', H'10C'-H'11F', H'150'-H'16F'
    __BADRAM H'185', H'187'-H'189', H'18C'-H'1EF'

;==========================================================================
;
;       Configuration Bits
;
;==========================================================================

_BODEN_ON                    EQU     H'3FFF'
_BODEN_OFF                   EQU     H'3FBF'
_CP_ALL                      EQU     H'03FF'
_CP_75                       EQU     H'17FF'
_CP_50                       EQU     H'2BFF'
_CP_OFF                      EQU     H'3FFF'
_DATA_CP_ON                  EQU     H'3EFF'
_DATA_CP_OFF                 EQU     H'3FFF'
_PWRTE_OFF                   EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LVP_ON                      EQU     H'3FFF'
_LVP_OFF                     EQU     H'3F7F'
_MCLRE_ON                    EQU     H'3FFF'
_MCLRE_OFF                   EQU     H'3FDF'
_ER_OSC_CLKOUT               EQU     H'3FFF'
_ER_OSC_NOCLKOUT             EQU     H'3FFE'
_INTRC_OSC_CLKOUT            EQU     H'3FFD'
_INTRC_OSC_NOCLKOUT          EQU     H'3FFC'
_EXTCLK_OSC                  EQU     H'3FEF'
_LP_OSC                      EQU     H'3FEC'
_XT_OSC                      EQU     H'3FED'
_HS_OSC                      EQU     H'3FEE'

	__CONFIG        _BODEN_OFF & _CP_OFF & _DATA_CP_OFF & _PWRTE_ON & _WDT_OFF & _LVP_OFF & _MCLRE_ON & _INTRC_OSC_NOCLKOUT

;==========================================================================
;       Variable Definition
;==========================================================================
;INPUTS
SW1		EQU     H'00'		;SW1 is triggering RA0
SW2		EQU     H'01'		;SW2 is triggering RA1
SW3		EQU     H'02'		;SW3 is triggering RA2
SW4		EQU     H'03'		;SW4 is triggering RA3
TIMER1		EQU	H'20'		;Used in delay routine
TIMER2		EQU	H'21'		; "	"	"	
PATERN		EQU	H'22'		;Pattern data for effect's
SFNC		EQU	H'23'		;selected function
VK1		EQU	H'24'		;help variable for key test
fnc3_hv		EQU	H'25'		;help for loop in fnc3
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;xxx              button service routine                                 xxx
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

key_test	btfss	PORTA,D'004'
		GOTO	KK2
;key pressed
		btfsc	VK1,D'000'	;if bit0 is set, key was pressed before as well
		goto	KK3		;no action, until key will be released again
;key was not pressed before; now we have to change the function...
		bsf	VK1,D'000'
		bsf	VK1,D'001'	;set bit 1 to indicate the change
		rlf	SFNC,1		;select next effect
		btfsc	SFNC,D'004'	;number of implemented options + 1
		goto	back_to_1
		goto	KK3
back_to_1	clrf	SFNC
		bsf	SFNC,D'000'	;first effect selected again
		goto	KK3

KK2		bcf	VK1,D'000'	;key not pressed now
KK3		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

DELAY_ROUTINE	MOVLW   D'255'
DELAY_ROUTINE2	MOVWF   TIMER2
DEL_LOOP1       MOVLW   D'255'

		CALL	key_test
		btfsc	VK1,D'001'
		goto	choose_effect

		MOVWF   TIMER1
DEL_LOOP2       DECFSZ  TIMER1,F
                GOTO    DEL_LOOP2
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP1
		RETLW   0


;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		MOVLW	B'00000111'	;Disable Comparator module's
		MOVWF	CMCON
		;
		BSF	STATUS,RP0	;Switch to register bank 1
					;Disable pull-ups
					;INT on rising edge
					;TMR0 to CLKOUT
					;TMR0 Incr low2high trans.
					;Prescaler assign to Timer0
					;Prescaler rate is 1:256
		MOVLW	B'11010111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		;
		CLRF	INTCON		;Disable interrupts
		MOVLW	B'00000000'
		MOVWF	TRISB		;RBx
		MOVLW	B'00110000'	;RA4 & RA5 - input
		MOVWF	TRISA
		BCF	STATUS,RP0	;Switch Back to reg. Bank 0

		CLRF	SFNC
		BSF	SFNC,D'000'	;select "fnc1"
		CLRF	VK1

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       main            XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

fnc1		MOVLW	B'11111111'
		MOVWF	PORTB
		MOVLW	B'11001111'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00000001'
		MOVWF	PORTB
		MOVLW	B'00000000'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		MOVLW	B'00000010'
		MOVWF	PORTB
		MOVLW	B'00000001'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00000100'
		MOVWF	PORTB
		MOVLW	B'00000010'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00001000'
		MOVWF	PORTB
		MOVLW	B'00000100'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00010000'
		MOVWF	PORTB
		MOVLW	B'00001000'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		MOVLW	B'00100000'
		MOVWF	PORTB
		MOVLW	B'01000000'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		MOVLW	B'01000000'
		MOVWF	PORTB
		MOVLW	B'10000000'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'10000000'
		MOVWF	PORTB
		MOVLW	B'00000000'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		goto	fnc1

;----------------------------------------------------------
fnc2		MOVLW	B'11111111'
		MOVWF	PORTB
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		CLRF	PORTB
		CLRF	PORTA
		CALL	DELAY_ROUTINE
		GOTO	fnc2

;----------------------------------------------------------
fnc3		MOVLW	B'00000001'
		MOVWF	PORTB
		MOVLW	B'00000000'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		MOVLW	B'00000011'
		MOVWF	PORTB
		MOVLW	B'00000001'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00000111'
		MOVWF	PORTB
		MOVLW	B'00000011'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00001111'
		MOVWF	PORTB
		MOVLW	B'00000111'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE


		MOVLW	B'00011111'
		MOVWF	PORTB
		MOVLW	B'00001111'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

		MOVLW	B'00111111'
		MOVWF	PORTB
		MOVLW	B'01001111'
		MOVWF	PORTA
		CALL	DELAY_ROUTINE

;finale...
		movlw	D'10'
		movwf	fnc3_hv		

fnc3_finloop1	MOVLW	B'01111111'
		MOVWF	PORTB
		MOVLW	B'11001111'
		MOVWF	PORTA

		MOVLW   D'100'
		CALL	DELAY_ROUTINE2

		MOVLW	B'10000000'
		MOVWF	PORTB
		MOVLW	B'00000000'
		MOVWF	PORTA

		MOVLW   D'100'
		CALL	DELAY_ROUTINE2

                DECFSZ  fnc3_hv,F
                GOTO    fnc3_finloop1

		goto	fnc3
;----------------------------------------------------------
fnc_sw_err	clrf	PORTA
		clrf	PORTB
		CALL	DELAY_ROUTINE
		movlw	B'00000001'
		movwf	PORTB
		CALL	DELAY_ROUTINE
		goto fnc_sw_err
;----------------------------------------------------------

choose_effect	bcf	VK1,D'001'
		btfsc	SFNC,D'000'
		goto	fnc1
		btfsc	SFNC,D'001'
		goto	fnc2
		btfsc	SFNC,D'002'
		goto	fnc3

		goto	fnc_sw_err

		END